==> 2.0.0.tar.gz: lib/mapping/server.dart <==
part of TeamSpeak;

abstract class TeamSpeakEntity {
  final Server server;
  final _Client _client;
  
  Future<TeamSpeakEntity> update();
  
  TeamSpeakEntity(Server this.server, _Client this._client);
}

/// Represents a virtual TeamSpeak-Server instance on a real TeamSpeak-Server
class Server {  
  final String host;
  final int port;
  final int serverId;
  final String username;
  String _password;
  final bool enableThrottling;
  
  String _version; String get version => _version;
  int _build; int get build => _build;
  String _platform; String get platform => _platform;
  
  /// Channels as cached after last listing. Channel-IDs are used as keys.
  Map<int, Channel> channels = new Map();
  
  /// Clients as cached after last listing. Database-IDs are used as keys.
  Map<int, User> users = new Map();
  
  _Client _client;
  
  Server(String this.host, {int this.port: 10011, String this.username: "", int this.serverId: 1, String password: "", this.enableThrottling: true}){
    _password = password;
  }
  
  /// Returns if this server-object is referencing the same TeamSpeak-Server as 
  /// another server-object. Keep in mind that this doesn't mean that both
  /// servers are connected using the same user.
  operator ==(Server serv){
    return serv.host == host && serv.port == port && serv.serverId == serverId;
  }
  
  /// Updates the information provided by this object. Is done automatically
  /// after connecting to the server.
  Future<Server> update(){
    return _client.send(new Command("version"))
        .then((Answer pAnswer){
          _version = pAnswer.items[0]["version"];
          _build = int.parse(pAnswer.items[0]["build"]);
          _platform = pAnswer.items[0]["platform"];
          return Future.wait([_updateChannels(),
                              _updateUsers()]);
        });
  }
  
  Future<List<User>> _updateUsers(){
    return _client.send(new Command("clientlist")).then((Answer answer){
      users.clear();
      final Map<int, User> ids = new Map();
      final List<User> result = answer
          .items
          .map((Map<String, String> user){
            final User u = new User(this, 
                _client, 
                int.parse(user["clid"]),
                int.parse(user["client_database_id"]));
            ids[u.database_id] = u;
            return u;
          })
          .toList(growable: false);
      users = ids;
      return Future.wait(result.map((User u) => u.update()));
    });
    
  }
  
  /// Returns a list of channels on this server. Does not cache.
  Future<List<Channel>> _updateChannels(){
    // Request channel listing from server
    return _client.send(new Command("channellist"))
        .then((Answer pAnswer){
          // Delete all existing channels
          channels.clear();
          final Map<int, Channel> ids = new Map();
          // Create a channel for each item
          final List<Channel> result = pAnswer
              .items
              .map((Map<String, String> channel){
                final Channel c = new Channel(this, _client, int.parse(channel["cid"]), int.parse(channel["pid"]));
                ids[c.id] = c;
                return c;
              })
              .toList(growable: false);
          // Update parents
          result.forEach((Channel c){
            if(c.hasParent){
              c.setParentChannel(ids[c.parentId]);
            }            
          });
          channels = ids;
          return Future.wait(result.map((Channel c) => c.update()));
        });
  }
  
  void addClient(Notification a){
    final int dbid = int.parse(a.parameters["client_database_id"]);
    final int clid = int.parse(a.parameters["clid"]);    
    if(!users.containsKey(dbid)){
      final User newuser = new User(this, _client, clid, dbid);
      newuser.update()
        .then((User user){
          if(!users.containsKey(dbid)){
            users[dbid] = user;
          }
        });
    }
  }
  
  void removeClient(Notification a){
    final int clid = int.parse(a.parameters["clid"]);
    final List<User> _users = users.values.where((User u) => u.clid == clid);
    if(_users.length == 1){
      final int dbid = _users.single.database_id;
      users[dbid].is_online = false;
      users.remove(dbid);
    }
  }
  
  /// Connects to the TeamSpeak-Server, logs in and runs [update].
  Future<Server> connect(){
    _client = new _Client(host, port);
    return _client.connect().then((dynamic dyn){
      if(username != ""){
        return _client.send(new Command("login", parameters: {"client_login_name": username, "client_login_password": _password}));
      } else {
        return new Future.value(false);
      }
    }).then((dynamic dyn){
      return _client.send(new Command("use", parameters: {"sid": serverId}));
    }).then((Answer answer){
      _client.send(new Command("servernotifyregister", parameters: {"event": "server"}))
        .then((Answer answer){
          _client.listenNotification("cliententerview").listen(addClient);
          _client.listenNotification("clientleftview").listen(removeClient);
        });
      return update();
    }).then((List data){
      return this;
    });
  }
}

==> 2.0.1.tar.gz: lib/mapping/server.dart <==
part of TeamSpeak;

abstract class TeamSpeakEntity {
  final Server server;
  final _Client _client;
  
  Future<TeamSpeakEntity> update();
  
  TeamSpeakEntity(Server this.server, _Client this._client);
}

/// Represents a virtual TeamSpeak-Server instance on a real TeamSpeak-Server
class Server {  
  final String host;
  final int port;
  final int serverId;
  final String username;
  String _password;
  final bool enableThrottling;
  
  String _version; String get version => _version;
  int _build; int get build => _build;
  String _platform; String get platform => _platform;
  
  /// Channels as cached after last listing. Channel-IDs are used as keys.
  Map<int, Channel> channels = new Map();
  
  /// Clients as cached after last listing. Database-IDs are used as keys.
  Map<int, User> users = new Map();
  
  Stream<UserEnterServerEvent> onUserEnter = _onUserEnter.stream; StreamController _onUserEnter = new StreamController.broadcast();
  Stream<UserLeaveServerEvent> onUserLeave = _onUserLeave.stream; StreamController _onUserLeave = new StreamController.broadcast();
  
  _Client _client;
  
  Server(String this.host, {int this.port: 10011, String this.username: "", int this.serverId: 1, String password: "", this.enableThrottling: true}){
    _password = password;
  }
  
  /// Returns if this server-object is referencing the same TeamSpeak-Server as 
  /// another server-object. Keep in mind that this doesn't mean that both
  /// servers are connected using the same user.
  operator ==(Server serv){
    return serv.host == host && serv.port == port && serv.serverId == serverId;
  }
  
  /// Updates the information provided by this object. Is done automatically
  /// after connecting to the server.
  Future<Server> update(){
    return _client.send(new Command("version"))
        .then((Answer pAnswer){
          _version = pAnswer.items[0]["version"];
          _build = int.parse(pAnswer.items[0]["build"]);
          _platform = pAnswer.items[0]["platform"];
          return Future.wait([_updateChannels(),
                              _updateUsers()]);
        });
  }
  
  Future<List<User>> _updateUsers(){
    return _client.send(new Command("clientlist")).then((Answer answer){
      users.clear();
      final Map<int, User> ids = new Map();
      final List<User> result = answer
          .items
          .map((Map<String, String> user){
            final User u = new User(this, 
                _client, 
                int.parse(user["clid"]),
                int.parse(user["client_database_id"]));
            ids[u.database_id] = u;
            return u;
          })
          .toList(growable: false);
      users = ids;
      return Future.wait(result.map((User u) => u.update()));
    });
    
  }
  
  /// Returns a list of channels on this server. Does not cache.
  Future<List<Channel>> _updateChannels(){
    // Request channel listing from server
    return _client.send(new Command("channellist"))
        .then((Answer pAnswer){
          // Delete all existing channels
          channels.clear();
          final Map<int, Channel> ids = new Map();
          // Create a channel for each item
          final List<Channel> result = pAnswer
              .items
              .map((Map<String, String> channel){
                final Channel c = new Channel(this, _client, int.parse(channel["cid"]), int.parse(channel["pid"]));
                ids[c.id] = c;
                return c;
              })
              .toList(growable: false);
          // Update parents
          result.forEach((Channel c){
            if(c.hasParent){
              c.setParentChannel(ids[c.parentId]);
            }            
          });
          channels = ids;
          return Future.wait(result.map((Channel c) => c.update()));
        });
  }
  
  void addClient(Notification a){
    final int dbid = int.parse(a.parameters["client_database_id"]);
    final int clid = int.parse(a.parameters["clid"]);    
    if(!users.containsKey(dbid)){
      final User newuser = new User(this, _client, clid, dbid);
      newuser.update()
        .then((User user){
          if(!users.containsKey(dbid)){
            users[dbid] = user;
            _onUserEnter.add(new UserEnterServerEvent(user));
          }
        });
    }
  }
  
  void removeClient(Notification a){
    final int clid = int.parse(a.parameters["clid"]);
    final List<User> _users = users.values.where((User u) => u.clid == clid);
    if(_users.length == 1){
      final User user = _users.single;
      final int dbid = user.database_id;
      user.is_online = false;
      users.remove(dbid);
      _onUserLeave.add(new UserLeaveServerEvent(user));
    }
  }
  
  /// Connects to the TeamSpeak-Server, logs in and runs [update].
  Future<Server> connect(){
    _client = new _Client(host, port);
    return _client.connect().then((dynamic dyn){
      if(username != ""){
        return _client.send(new Command("login", parameters: {"client_login_name": username, "client_login_password": _password}));
      } else {
        return new Future.value(false);
      }
    }).then((dynamic dyn){
      return _client.send(new Command("use", parameters: {"sid": serverId}));
    }).then((Answer answer){
      _client.send(new Command("servernotifyregister", parameters: {"event": "server"}))
        .then((Answer answer){
          _client.listenNotification("cliententerview").listen(addClient);
          _client.listenNotification("clientleftview").listen(removeClient);
        });
      return update();
    }).then((List data){
      return this;
    });
  }
}
